;;; ARC_CDL, ARC_DCL et ARC_DEL (gile)
;;; Pour tracer des arcs d'aprs leur longueur.

;;;==================================== Sous routines ====================================;;;

;;; ArcCorde->Angle (gile)
;;; Retourne l'angle d'un arc d'aprs sa longueur et sa corde (mthode de Newton)

(defun ArcCorde->Angle (arc corde / k c)
  (setq	k (/ corde arc)
	c (sqrt (- 6 (* 6 k)))
  )
  (repeat 6
    (setq c (- c (/ (- (sin c) (* k c)) (- (cos c) k))))
  )
  (* 2 c)
)

;;; GETLONG (gile)
;;; Demande  l'utilisateur de spcifier une longueur, soit avec GETDIST, soit en
;;; slectionnant un objet, soit en utilisant la calculatrice gomtrique.
;;; NOTA : init_lst est la liste des arguments pour la fonction initget
;;;        ex: (GETLONG '(7 "Objet Calcul") "\nSpcifiez la longueur ou [Objet/Calcul]: ")

(defun GETLONG (init_lst msg / obj long)
  (apply 'initget init_lst)
  (setq	long (getdist msg)
  )
  (cond
    ((= long "Objet")
     (setq long (VL-LONGOBJT (car (entsel))))
     (while (not (numberp long))
       (princ "\nLa longueur de cet objet n'est pas dfinie.")
       (setq long (VL-LONGOBJT (car (entsel))))
     )
    )
    ((= long "Calcul")
     (if (not (member "geomcal.arx" (arx)))
       (arxload "geomcal")
     )
     (setq long (C:CAL))
    )
  )
  long
)

;;; VL-LONGOBJT (gile)
;;;  Retourne la longueur ou le primtre d'un objet.
;;; Accepte un argument de type ename ou vla-object.

(defun VL-LONGOBJT (obj)
  (if (= (type obj) 'ENAME)
    (setq obj (vlax-ename->vla-object obj))
  )
  (if (and (= (type obj) 'VLA-OBJECT)
	   (not	(vl-catch-all-error-p
		  (vl-catch-all-apply
		    'vlax-curve-getEndParam
		    (list obj)
		  )
		)
	   )
      )
    (vlax-curve-getDistAtParam obj (vlax-curve-getEndParam obj))
  )
)

;;;================================== Fonctions d'appel ==================================;;;

;;; C:ARC_CDL (gile)
;;; Cre un arc  partir d'un centre, d'un point de dpart et de la longueur de l'arc

(defun c:arc_cdl (/ cen dep long AcDoc Space ray ang ang0 ang1 temp ucszdir)
  (vl-load-com)
  (initget 1)
  (setq cen (getpoint "\nSpcifiez le centre de l'arc: "))
  (initget 1)
  (setq	dep	(getpoint cen "\nSpcifiez le point de dpart de l'arc: ")
	long	(GETLONG
		  '(7 "Objet Calcul")
		  "\nSpcifiez la longueur de l'arc ou [Objet/Calcul]: "
		)
	AcDoc	(vla-get-ActiveDocument (vlax-get-acad-object))
	Space	(if (= 1 (getvar "CVPORT"))
		  (vla-get-PaperSpace AcDoc)
		  (vla-get-ModelSpace acDoc)
		)
	ucszdir	(trans '(0 0 1) 1 0 0)
	ray	(distance cen dep)
	ang0	(angle (trans cen 1 ucszdir) (trans dep 1 ucszdir))
	ang	(/ long ray)
  )
  (initget "Horaire Trigonomtrique")
  (if
    (= (getkword
	 "\nEntrez le sens de rotation [Horaire/Trigonomtrique] <T>: "
       )
       "Horaire"
    )
     (setq temp	ang0
	   ang0	(- ang0 ang)
	   ang1	temp
	   temp	nil
     )
     (setq ang1 (+ ang0 ang))
  )
  (vla-StartUndoMark acDoc)
  (vla-put-Normal
    (vla-addArc	Space
		(vlax-3d-point (trans cen 1 0))
		ray
		ang0
		ang1
    )
    (vlax-3d-point ucszdir)
  )
  (vla-EndUndoMark acDoc)
  (princ)
)

;;; C:ARC_DCL (gile)
;;; Cre un arc  partir d'un point de dpart, d'un centre et de la longueur de l'arc

(defun c:arc_dcl (/ cen dep long AcDoc Space ray ang ang0 ang1 temp ucszdir)
  (vl-load-com)
  (initget 1)
  (setq dep (getpoint "\nSpcifiez le point de dpart de l'arc: "))
  (initget 1)
  (setq	cen	(getpoint dep "\nSpcifiez le centre de l'arc: ")
	long	(GETLONG
		  '(7 "Objet Calcul")
		  "\nSpcifiez la longueur de l'arc ou [Objet/Calcul]: "
		)
	AcDoc	(vla-get-ActiveDocument (vlax-get-acad-object))
	Space	(if (= 1 (getvar "CVPORT"))
		  (vla-get-PaperSpace AcDoc)
		  (vla-get-ModelSpace acDoc)
		)
	ucszdir	(trans '(0 0 1) 1 0 0)
	ray	(distance cen dep)
	ang0	(angle (trans cen 1 ucszdir) (trans dep 1 ucszdir))
	ang	(/ long ray)
  )
  (initget "Horaire Trigonomtrique")
  (if
    (= (getkword
	 "\nEntrez le sens de rotation [Horaire/Trigonomtrique] <T>: "
       )
       "Horaire"
    )
     (setq temp	ang0
	   ang0	(- ang0 ang)
	   ang1	temp
	   temp	nil
     )
     (setq ang1 (+ ang0 ang))
  )
  (vla-StartUndoMark acDoc)
  (vla-put-Normal
    (vla-addArc	Space
		(vlax-3d-point (trans cen 1 0))
		ray
		ang0
		ang1
    )
    (vlax-3d-point ucszdir)
  )
  (vla-EndUndoMark acDoc)
  (princ)
)

;;; C:ARC_DEL (gile)
;;; Cre un arc  partir d'un point de dpart, d'un point d'arrive et de la longueur de l'arc

(defun c:arc_del
       (/ dep ext long AcDoc Space ucszdir cord ang ray mid cen ang0 ang1)
  (vl-load-com)
  (initget 1)
  (setq dep (getpoint "\nSpcifiez le point de dpart de l'arc: "))
  (initget 1)
  (setq	ext  (getpoint dep "\nSpcifiez l'extrmit de l'arc: ")
	long
	     (GETLONG
	       '(7 "Objet Calcul")
	       "\nSpcifiez la longueur de l'arc ou [Objet/Calcul]: "
	     )
  )
  (if (< long (distance dep ext))
    (princ "Erreur: argument incorrect: corde > longueur")
    (progn
      (setq AcDoc   (vla-get-ActiveDocument (vlax-get-acad-object))
	    Space   (if	(= 1 (getvar "CVPORT"))
		      (vla-get-PaperSpace AcDoc)
		      (vla-get-ModelSpace acDoc)
		    )
	    ucszdir (trans '(0 0 1) 1 0 0)
	    cord    (distance dep ext)
	    ang	    (/ (ArcCorde->Angle long cord) 2)
	    ray	    (abs (/ cord 2 (sin ang)))
	    mid	    (mapcar '(lambda (p1 p2) (/ (+ p1 p2) 2.0)) dep ext)
      )
      (if (equal (/ pi 2) ang 1e-009)
	(setq cen mid
	      ray (/ cord 2)
	)
	(setq
	  cen (polar mid (+ (angle dep ext) (/ pi 2)) (* ray (cos ang)))
	)
      )
      (foreach pt '(cen dep ext)
	(set pt (trans (eval pt) 1 ucszdir))
      )
      (setq ang0 (angle cen dep)
	    ang1 (angle cen ext)
      )
      (vla-StartUndoMark acDoc)
      (vla-put-Normal
	(vla-addArc Space
		    (vlax-3d-point (trans cen ucszdir 0))
		    ray
		    ang0
		    ang1
	)
	(vlax-3d-point ucszdir)
      )
      (vla-EndUndoMark acDoc)
    )
  )
  (princ)
)