;;; DIV3D et MES3D - Gilles Chanteau - 25/08/06
;;; Correction le 10/05/07
;;;
;;; Fonctionnement similaire aux commandes DIVISER (_DIVIDE) et MESURER (_MEASURE)
;;; Les plus :
;;; - L'option "Bloc" permet l'insertion de bloc sur des objets "mesurables" quels que
;;; soient le SCO de l'objet et le SCU courant.
;;; - Une option supplmentaire : "Plan de rfrence pour l'insertion du bloc ?"
;;; permet de choisir si le bloc est insr par rapport au plan XY du SCU courant
;;; [Scu] ou par rapport  l'objet [Objet].
;;; - Le choix du bloc se fait dans la liste droulante d'une boite de dialogue.
;;; - La longueur de l'objet slectionne est affiche sur la ligne de commande.


;;; Getspace Retourne l'espace courant (Modle ou Papier)

(defun getSpace	()
  (if (= (getvar "CVPORT") 1)
    (vla-get-PaperSpace
      (vla-get-ActiveDocument (vlax-get-acad-object))
    )
    (vla-get-ModelSpace
      (vla-get-ActiveDocument (vlax-get-acad-object))
    )
  )
)

;;; Getblock (gile) 03/11/07
;;; Retourne le nom du bloc entr ou choisi par l'utilisateur 
;;; dans une liste droulante de la boite de dialogue ou depuis la boite
;;; de dialogue standard d'AutoCAD
;;; Argument : le titre (string) ou nil (dfaut : "Choisir un bloc")

(defun getblock	(titre / bloc n lst tmp file what_next dcl_id nom)
  (while (setq bloc (tblnext "BLOCK" (not bloc)))
    (setq lst (cons (cdr (assoc 2 bloc)) lst)
    )
  )
  (setq	lst  (acad_strlsort
	       (vl-remove-if
		 (function (lambda (n) (= (substr n 1 1) "*")))
		 lst
	       )
	     )
	tmp  (vl-filename-mktemp "Tmp.dcl")
	file (open tmp "w")
  )
  (write-line
    (strcat
      "getblock:dialog{label="
      (cond (titre (vl-prin1-to-string titre))
	    ("\"Choisir un bloc\"")
      )
      ";initial_focus=\"bl\";:boxed_column{
      :row{:text{label=\"Slectionner\";alignment=left;}
      :button{label=\">>\";key=\"sel\";alignment=right;fixed_width=true;}}
      spacer;
      :column{:button{label=\"Parcourir...\";key=\"wbl\";alignment=right;fixed_width=true;}}
      :column{:text{label=\"Nom :\";alignment=left;}}
      :edit_box{key=\"tp\";edit_width=25;}
      :popup_list{key=\"bl\";edit_width=25;}spacer;}
      spacer;
      ok_cancel;}"
    )
    file
  )
  (close file)
  (setq dcl_id (load_dialog tmp))
  (setq what_next 2)
  (while (>= what_next 2)
    (if	(not (new_dialog "getblock" dcl_id))
      (exit)
    )
    (start_list "bl")
    (mapcar 'add_list lst)
    (end_list)
    (if	(setq n	(vl-position
		  (strcase (getvar "INSNAME"))
		  (mapcar 'strcase lst)
		)
	)
      (setq nom (nth n lst))
      (setq nom	(car lst)
	    n	0
      )
    )
    (set_tile "bl" (itoa n))
    (action_tile "sel" "(done_dialog 5)")
    (action_tile "bl" "(setq nom (nth (atoi $value) lst))")
    (action_tile "wbl" "(done_dialog 3)")
    (action_tile "tp" "(setq nom $value) (done_dialog 4)")
    (action_tile
      "accept"
      "(setq nom (nth (atoi (get_tile \"bl\")) lst)) (done_dialog 1)"
    )
    (setq what_next (start_dialog))
    (cond
      ((= what_next 3)
       (if (setq nom (getfiled "Slectionner un fichier" "" "dwg" 0))
	 (setq what_next 1)
	 (setq what_next 2)
       )
      )
      ((= what_next 4)
       (cond
	 ((not (read nom))
	  (setq what_next 2)
	 )
	 ((tblsearch "BLOCK" nom)
	  (setq what_next 1)
	 )
	 ((findfile (setq nom (strcat nom ".dwg")))
	  (setq what_next 1)
	 )
	 (T
	  (alert (strcat "Le fichier \"" nom "\" est introuvable."))
	  (setq	nom nil
		what_next 2
	  )
	 )
       )
      )
      ((= what_next 5)
       (if (and	(setq ent (car (entsel)))
		(= "INSERT" (cdr (assoc 0 (entget ent))))
	   )
	 (setq nom	 (cdr (assoc 2 (entget ent)))
	       what_next 1
	 )
	 (setq what_next 2)
       )
      )
      ((= what_next 0)
       (setq nom nil)
      )
    )
  )
  (unload_dialog dcl_id)
  (vl-file-delete tmp)
  nom
)

;;; V^V (gile)
;;; Retourne le produit vectoriel (vecteur) de deux vecteurs

(defun v^v (v1 v2)
  (if (inters '(0 0 0) v1 '(0 0 0) v2)
    (list (- (* (cadr v1) (caddr v2)) (* (caddr v1) (cadr v2)))
	  (- (* (caddr v1) (car v2)) (* (car v1) (caddr v2)))
	  (- (* (car v1) (cadr v2)) (* (cadr v1) (car v2)))
    )
  )
)

;;; NORM_3PTS (gile)
;;; retourne le vecteur normal du plan dfini par 3 points

(defun norm_3pts (org xdir ydir / norm)
  (foreach v '(xdir ydir)
    (set v (mapcar '- (eval v) org))
  )
  (mapcar '(lambda (x) (/ x (distance '(0 0 0) norm)))
	  (setq norm (v^v xdir ydir))
  )
)

;;; Transpose une matrice -Doug Wilson-

(defun trp (m)
  (apply 'mapcar (cons 'list m))
)

;;; Ang<2pi (gile)
;;; Retourne l'angle,  2*k*pi prs, compris entre 0 et 2*pi

(defun ang<2pi (ang)
  (if (and (<= 0 ang) (< ang (* 2 pi)))
    ang
    (ang<2pi (rem (+ ang (* 2 pi)) (* 2 pi)))
  )
)

;;; DivMesInsert (gile)
;;; Insre le bloc (ou le point) suivant les options choisies

(defun DivMesInsert (/ norm deriv2 ang mat ydir)
  (if (/= scu "Scu")
    (cond
      ((member (vla-get-ObjectName obj)
	       '("AcDbArc"	     "AcDbCircle"
		 "AcDbEllipse"	     "AcDbPolyline"
		 "AcDb2dPolyline"
		)
       )
       (setq
	 norm (vlax-safearray->list
		(vlax-variant-value (vla-get-Normal obj))
	      )
       )
      )
      ((not (member (vla-get-ObjectName obj)
		    '("AcDbLine" "AcDb3dPolyline")
	    )
       )
       (setq
	 deriv2
	  (vlax-curve-getSecondDeriv
	    obj
	    ins_par
	  )
       )
       (setq ang
	      (ang<2pi (- (angle '(0 0 0) deriv2) (angle '(0 0 0) deriv1))
	      )
       )
       (if
	 (< ang pi)
	  (setq norm (norm_3pts '(0 0 0) deriv1 deriv2))
	  (setq norm (norm_3pts '(0 0 0) deriv2 deriv1))
       )
      )
    )
  )
  (if nom
    (progn
      (if (= scu "Scu")
	(progn
	  (if (= algn "Non")
	    (setq
	      ang
	       (angle '(0 0 0)
		      (trans (getvar "UCSXDIR") 0 ucszdir)
	       )
	    )
	    (setq ang
		   (angle '(0 0 0) (trans deriv1 0 ucszdir))
	    )
	  )
	  (setq
	    mat	(mapcar
		  '(lambda (v) (trans v 0 ucszdir))
		  (list	(list (cos ang) (- (sin ang)) 0)
			(list (sin ang) (cos ang) 0)
			'(0 0 1)
		  )
		)
	  )
	)
	(if (= algn "Non")
	  (setq	ydir (norm_3pts '(0 0 0) norm (getvar "ucsxdir"))
		mat  (trp
		       (cons
			 (norm_3Pts '(0 0 0) ydir norm)
			 (cons
			   ydir
			   (cons norm mat)
			 )
		       )
		     )
	  )
	  (setq
	    mat
	     (trp
	       (list (mapcar '(lambda (x)
				(/ x (distance '(0 0 0) deriv1))
			      )
			     deriv1
		     )
		     (norm_3pts '(0 0 0) norm deriv1)
		     norm
	       )
	     )
	  )
	)
      )
      (vla-TransformBy
	(vla-InsertBlock
	  (getspace)
	  (vlax-3d-point '(0 0 0))
	  nom
	  1
	  1
	  1
	  0
	)
	(vlax-tmatrix
	  (append
	    (mapcar
	      '(lambda (v1 v2)
		 (append v1 (list v2))
	       )
	      mat
	      ins_pt
	    )
	    (list '(0 0 0 1))
	  )
	)
      )
      (setvar "INSNAME" (vl-filename-base nom))
    )
    (vla-addPoint (getspace) (vlax-3d-point ins_pt))
  )
)

;;; Std_vl_err Redfinition de *error*

(defun std_vl_err (msg)
  (if (= msg "Fonction annule")
    (princ)
    (princ (strcat "\nErreur: " msg))
  )
  (vla-EndUndoMark
    (vla-get-ActiveDocument (vlax-get-acad-object))
  )
  (setq	*error*	m:err
	m:err nil
  )
  (princ)
)

;;;******************************************************************;;;
;;; Div3D (gile)

(defun c:div3D (/	AcDoc	ucszdir	ent	obj	pick	long
		nb	nom	algn	scu	dist	ins_pt	ins_par
		deriv1
	       )
  (vl-load-com)
  (setq	m:err	*error*
	*error*	std_vl_err
  )
  (setq	AcDoc	(vla-get-ActiveDocument (vlax-get-acad-object))
	ucszdir	(trans '(0 0 1) 1 0 T)
  )
  (sssetfirst nil nil)
  (if (vl-catch-all-error-p
	(vl-catch-all-apply
	  'vla-getEntity
	  (list	(vla-get-Utility AcDoc)
		'obj
		'pick
		"\nChoix de l'objet  diviser: "
	  )
	)
      )
    (prompt "*Incorrect*")
    (progn
      (if (vl-catch-all-error-p
	    (vl-catch-all-apply 'vlax-curve-getEndParam (list obj))
	  )
	(prompt "\nL'objet ne peut pas tre divis.*Incorrect*")
	(progn
	  (setq	long (vlax-curve-getDistAtParam
		       obj
		       (vlax-curve-getEndParam obj)
		     )
	  )
	  (prompt (strcat "\nLongueur de l'objet: " (rtos long)))
	  (while (not (> 3268 nb 1))
	    (setq
	      nb
	       (getint "\nEntrez le nombre de segments ou <Bloc>: ")
	    )
	    (if	nb
	      (if (not (> 3268 nb 1))
		(prompt
		  "\nNcessite un entier entre 2 et 32767, ou une option."
		)
	      )
	      (progn
		(setq nom (getblock nil))
		(if nom
		  (setq nb 2)
		  (setq nb 0)
		)
	      )
	    )
	  )
	  (if nom
	    (progn
	      (initget "Oui Non")
	      (setq algn
		     (getkword
		       "\nAligner le bloc avec l'objet ? [Oui/Non] <Oui>: "
		     )
		    nb 0
	      )
	      (if (not (member (vla-get-ObjectName obj)
			       '("AcDbLine" "AcDb3dPolyline")
		       )
		  )
		(progn
		  (initget "Objet Scu")
		  (setq	scu
			 (getkword
			   "\nPlan de rfrence pour l'insertion du bloc ? [Objet/Scu] <Objet>: "
			 )
		  )
		)
		(setq scu "Scu")
	      )
	      (while (not (> 3268 nb 1))
		(setq nb (getint "\nEntrez le nombre de segments: "))
		(if (not (> 3268 nb 1))
		  (prompt
		    "\nNcessite un entier entre 2 et 32767."
		  )
		)
	      )
	    )
	  )
	  (setq dist (/ long nb))
	  (vla-StartUndoMark AcDoc)
	  (repeat (setq nb (1- nb))
	    (setq ins_pt  (vlax-curve-getPointAtDist obj (* nb dist))
		  ins_par (vlax-curve-getParamAtDist obj (* nb dist))
		  deriv1  (vlax-curve-getFirstDeriv
			    obj
			    ins_par
			  )
	    )
	    (DivMesInsert)
	    (setq nb (1- nb))
	  )
	  (if (vlax-curve-isClosed obj)
	    (progn
	      (setq ins_pt  (vlax-curve-getStartPoint obj)
		    ins_par (vlax-curve-getStartParam obj)
		    deriv1  (vlax-curve-getFirstDeriv
			      obj
			      ins_par
			    )
	      )
	      (DivMesInsert)
	    )
	  )
	  (vla-EndUndoMark AcDoc)
	)
      )
    )
  )
  (setq	*error*	m:err
	m:err nil
  )
  (princ)
)

;;;******************************************************************;;;
;;; Mes3D (gile)

(defun c:mes3D
	       (/ AcDoc	ucszdir	obj pick long dist nom algn scu	nb rest
		ins_pt deriv1)
  (vl-load-com)
  (setq	m:err	*error*
	*error*	std_vl_err
  )
  (setq	AcDoc	(vla-get-ActiveDocument (vlax-get-acad-object))
	ucszdir	(trans '(0 0 1) 1 0 T)
  )
  (sssetfirst nil nil)
  (if (vl-catch-all-error-p
	(vl-catch-all-apply
	  'vla-getEntity
	  (list	(vla-get-Utility AcDoc)
		'obj
		'pick
		"\nChoix de l'objet  mesurer: "
	  )
	)
      )
    (prompt "*Incorrect*")
    (progn
      (if (vl-catch-all-error-p
	    (vl-catch-all-apply 'vlax-curve-getEndParam (list obj))
	  )
	(prompt "\nL'objet ne peut pas tre mesur.*Incorrect*")
	(progn
	  (setq	pick (vlax-curve-getClosestPointToProjection
		       obj
		       (trans (vlax-SafeArray->list pick) 1 0)
		       (mapcar '-
			       (trans (getvar "viewdir") 1 0)
			       (trans '(0 0 0) 1 0)
		       )
		     )
		long (vlax-curve-getDistAtParam
		       obj
		       (vlax-curve-getEndParam obj)
		     )
	  )
	  (prompt (strcat "\nLongueur de l'objet: " (rtos long)))
	  (while (and (not (numberp dist)) (not nom))
	    (if	(not (setq dist
			    (getdist
			      "\nSpcifiez la longueur du segment ou <Bloc>: "
			    )
		     )
		)
	      (setq nom (getblock nil))
	    )
	  )
	  (if nom
	    (progn
	      (initget "Oui Non")
	      (setq algn
		     (getkword
		       "\nAligner le bloc avec l'objet ? [Oui/Non] <Oui>: "
		     )
	      )
	      (if (not (member (vla-get-ObjectName obj)
			       '("AcDbLine" "AcDb3dPolyline")
		       )
		  )
		(progn
		  (initget "Objet Scu")
		  (setq	scu
			 (getkword
			   "\nPlan de rfrence pour l'insertion du bloc ? [Objet/Scu] <Objet>: "
			 )
		  )
		)
		(setq scu "Scu")
	      )
	      (initget 7)
	      (setq
		dist (getdist "\nSpcifiez la longueur du segment: ")
	      )
	    )
	  )
	  (if (< long dist)
	    (prompt "\nL'objet n'est pas assez long.")
	    (progn
	      (setq nb (fix (/ long dist)))
	      (if
		(< (/ long 2) (vlax-curve-getDistAtPoint obj pick))
		 (setq rest (rem long dist))
		 (setq rest nil)
	      )
	      (vla-StartUndoMark AcDoc)
	      (repeat nb
		(setq ins_pt  (if rest
				(vlax-curve-getPointAtDist
				  obj
				  (+ (* (1- nb) dist) rest)
				)
				(vlax-curve-getPointAtDist obj (* nb dist))
			      )
		      ins_par (if rest
				(vlax-curve-getParamAtDist
				  obj
				  (+ (* (1- nb) dist) rest)
				)
				(vlax-curve-getParamAtDist obj (* nb dist))
			      )
		      deriv1  (vlax-curve-getFirstDeriv obj ins_par)
		)
		(DivMesInsert)
		(setq nb (1- nb))
	      )
	      (vla-EndUndoMark AcDoc)
	    )
	  )
	)
      )
    )
  )
  (setq	*error*	m:err
	m:err nil
  )
  (princ)
)