;; Manipulation des listes

;; BUTLAST
;; Retourne la liste sans le dernier lment

(defun butlast (lst)
  (reverse (cdr (reverse lst)))
)

;; CAR2LAST
;; Dplace le premier lment  la fin de la liste

(defun car2last	(lst)
  (reverse (cons (car lst) (reverse (cdr lst))))
)

;; REMOVE_DOUBLES
;; Suprime tous les doublons d'une liste

(defun remove_doubles (lst)
  (if lst
    (cons (car lst) (remove_doubles (vl-remove (car lst) lst)))
  )
)

;; REMOVE-I
;; Retourne la liste prive de l'lment  l'indice spcifi
;; (premier lment = 0)

(defun remove-i (ind lst)
  (if (or (zerop ind) (null lst))
    (cdr lst)
    (cons (car lst) (remove-i (1- ind) (cdr lst)))
  )
)

;; REMOVE_FT
;; Retourne la liste prive des lements du premier indice
;; au second indice spcifi (premier lment = 0)

(defun remove_ft (lst from to)
  (cond
    ((or (null lst) (zerop to)) (cdr lst))
    ((< 0 from)
     (cons (car lst) (remove_ft (cdr lst) (1- from) (1- to)))
    )
    (T (remove_ft (cdr lst) (1- from) (1- to)))
  )
)

;; REMOVE-ELE
;; Retourne la liste sans la premire occurence de l'expression

(defun remove-ele (ele lst)
  (if (equal ele (car lst))
    (cdr lst)
    (cons (car lst) (remove-ele ele (cdr lst)))
  )
)

;; SUBST-I
;; Remplace l'lment  l'indice spcifi par un nouvel lment

(defun subst-i (new ind lst)
  (if (or (zerop ind) (null lst))
    (cons new (cdr lst))
    (cons (car lst) (subst-i new (1- ind) (cdr lst)))
  )
)

;; COMMON
;;; Retourne la liste des lments communs  lst1 et lst2
;; Retourne la liste des lments communs  lst1 et lst2

(defun common (l1 l2)
  (if l1
    (if	(member (car l1) l2)
      (cons (car l1) (common (cdr l1) l2))
      (common (cdr l1) l2)
    )
  )
)

;; EXCLUSIVE
;; Retourne une liste contenant les lments appartenant exclusivement  l1

(defun exclusive (l1 l2)
  (if l1
    (if	(member (car l1) l2)
      (exclusive (cdr l1) l2)
      (cons (car l1) (exclusive (cdr l1) l2))
    )
  )
)

;; EQUAL_EVERY
;; value si tous les membres d'une liste sont gaux  "fuzz" prs

(defun EQUAL_EVERY (lst fuzz)
  (vl-every (function (lambda (x) (equal x (car lst) fuzz)))
	    (cdr lst)
  )
)

;; TRUNC
;; Retourne la liste tronque  partir de la premire occurrence de l'expression
;; (liste complmentaire de celle retourne par MEMBER)

(defun trunc (expr lst)
  (if (and lst
	   (not (equal (car lst) expr))
      )
    (cons (car lst) (trunc expr (cdr lst)))
  )
)

;; TRUNC-IF
;; Retourne la liste tronque  partir de la premire occurrence qui
;; retourne T  la fonction (complmentaire de celle retourne par VL-MEMBER-IF)

(defun trunc-if	(fun lst)
  (if (and lst
	   (not ((eval fun) (car lst)))
      )
    (cons (car lst) (trunc-if fun (cdr lst)))
  )
)

;; MEMBER-FUZZ
;; Comme MEMBER avec une tolrance dans la comparaison

(defun member-fuzz (expr lst fuzz)
  (while (and lst (not (equal (car lst) expr fuzz)))
    (setq lst (cdr lst))
  )
  lst
)

;; TRUNC-FUZZ 
;; Comme TRUNC avec une tolrance dans la comparaison

(defun trunc-fuzz (expr lst fuzz)
  (if (and lst
	   (not (equal (car lst) expr))
      )
    (cons (car lst) (trunc-fuzz expr (cdr lst) fuzz))
  )
)

;;; SUBLIST
;;;  Retourne une sous-liste
;;;
;;; Arguments
;;; lst : une liste
;;; start : l'index de dpart de la sous liste (premier lment = 0)
;;; leng : la longueur (nombre d'lments) de la sous-liste (ou nil)
;;;
;;; Exemples :
;;; (sublist '(1 2 3 4 5 6) 2 2) -> (3 4)
;;; (sublist '(1 2 3 4 5 6) 2 nil) -> (3 4 5 6)

(defun sublist (lst start leng / n r)
  (if (or (not leng) (< (- (length lst) start) leng))
    (setq leng (- (length lst) start))
  )
  (setq n (+ start leng))
  (repeat leng
    (setq r (cons (nth (setq n (1- n)) lst) r))
  )
)

;; SPLIT-LIST
;;; Retourne une liste de sous-listes
;;;
;; Arguments
;; - lst : la liste  fractionner
;; - num : un entier, le nombre d'lments des sous listes
;; Exemples :
;; (split-list '(1 2 3 4 5 6 7 8) 2) -> ((1 2) (3 4) (5 6) (7 8))
;; (split-list '(1 2 3 4 5 6 7 8) 3) -> ((1 2 3) (4 5 6) (7 8))

(defun split-list (lst n)
  (if lst
    (cons (sublist lst 0 n)
	  (split-list (sublist lst n nil) n)
    )
  )
)

;; COMBI (gile)
;; Retourne la liste de toutes les combinaisons possibles
;; (combi '(1 2 3)) -> ((1) (1 2) (1 3) (1 2 3) (2) (2 3) (3))

(defun combi (lst / subr first tmp1 tmp2 tmp3 ret)

  (defun subr (l1 l2)
    (if	l2
      (cons (append l1 (list (car l2)))
	    (subr l1 (cdr l2))
      )
    )
  )

  (while lst
    (setq first	(list (car lst))
	  tmp1	(subr first (cdr lst))
	  tmp2	(cons first tmp1)
    )
    (while tmp1
      (setq tmp3 (subr (car tmp1) (cdr (member (last (car tmp1)) lst)))
	    tmp1 (cdr (append tmp1 tmp3))
	    tmp2 (append tmp2 tmp3)
      )
    )
    (setq ret (append ret tmp2)
	  lst (cdr lst)
    )
  )
  ret
)

;; COMBI3 (gile)
;; Retourne la liste de toutes les combinaisons par 3
;; (combi3 '(1 2 3 4)) -> ((1 2 3) (1 2 4) (1 3 4) (2 3 4))

(defun combi3 (l / subr1 subr2)

  (defun subr1 (e1 e2 l)
    (if	l
      (cons (list e1 e2 (car l))
	    (subr1 e1 e2 (cdr l))
      )
    )
  )

  (defun subr2 (e l)
    (if	(cdr l)
      (append (subr1 e (car l) (cdr l))
	      (subr2 e (cdr l))
      )
    )
  )

  (if (cddr l)
    (append (subr2 (car l) (cdr l))
	    (combi3 (cdr l))
    )
  ) 
)

;; SORT (gile)
;; Trie une liste  l'aide d'une fonction de comparaison
;; fonctionne comme vl-sort sans supprimer les doublons
(defun sort (lst fun / merge tmp)

  (defun merge (l1 l2)
    (if	(and l1 l2)
      (if (fun (car l1) (car l2))
	(cons (car l1) (merge (cdr l1) l2))
	(cons (car l2) (merge l1 (cdr l2)))
      )
      (if l1
	l1
	l2
      )
    )
  )

  (setq	lst (mapcar 'list lst)
	fun (eval fun)
  )
  (while (cdr lst)
    (setq tmp lst
	  lst nil
    )
    (while (cdr tmp)
      (setq lst	(cons (merge (car tmp) (cadr tmp)) lst)
	    tmp	(cddr tmp)
      )
    )
    (and tmp (setq lst (cons (car tmp) lst)))
  )
  (car lst)
)

;; STR2LST
;; Transforme un chaine avec sparateur en liste de chaines
;;
;; Arguments
;; str : la chaine  transformer en liste
;; sep : le sparateur

;;
;; Exemples
;; (str2lst "a b c" " ") -> ("a" "b" "c")
;; (str2lst "1,2,3" ",") -> ("1" "2" "3")
;; (mapcar 'read (str2lst "1,2,3" ",")) -> (1 2 3)

(defun str2lst (str sep / pos)
  (if (setq pos (vl-string-search sep str))
    (cons (substr str 1 pos)
	  (str2lst (substr str (+ (strlen sep) pos 1)) sep)
    )
    (list str)
  )
)

;; LST2STR
;; Concatne une liste et un sparateur en une chaine
;;
;; Arguments
;; lst : la liste  transformer en chaine
;; sep : le sparateur
;;
;; Exemples
;; (lst2str '(1 2 3) ",") -> "1,2,3"
;; (lst2str '("a" "b" "c") " ") -> "a b c"

(defun lst2str (lst sep)
  (if (cadr lst)
    (strcat (vl-princ-to-string (car lst))
	    sep
	    (lst2str (cdr lst) sep)
    )
    (vl-princ-to-string (car lst))
  )
)