;;; C:LONG_OBJT (gile) 10/04/05
;;; Modifie la longueur totale en fonction de la longueur (ou du primtre) de l'objet slectionn.

;;; Redfinition de *error*
(defun LO_ERR (msg)
  (if (/= msg "quitter / sortir abandon")
    (princ (strcat "\nErreur: " msg))
  )
  (command)
  (command "_undo" "_end")
  (if SavedSysVarLst
    (mapcar 'eval SavedSysVarLst)
  )
  (setq	*error*	m:err
	m:err nil
  )
  (princ)
)

;;; SAVE&SET_VAR (gile)
;;; Enregistre la valeur initiale de la variable systme dans une liste
;;; et lui attribue sa nouvelle valeur (si val est non nil)
;;;
;;; ex: (SAVE&SET_VAR "autosnap" nil) (SAVE&SET_VAR "orthomode" 1)
;;; -> SavedSysVarLst : (("orthomode" . 0) ("autosnap" . 63))
;;; -> (getvar "orthomode") : 1
;;; -> (getvar "autosnap") : 55 (63 - 8 : le reprage polaire est dsactiv)
;;; Pour restaurer les variables : (mapcar 'eval SavedSysVarLst)

(defun SAVE&SET_VAR (var val)
  (cond
    ((not (getvar var))
     (princ (strcat "\nErreur: variable AutoCAD rejete: " var))
     (princ)
    )
    (T
     (setq SavedSysVarLst
	    (cons (list 'setvar var (getvar var)) SavedSysVarLst)
     )
     (if val
       (setvar var val)
     )
    )
  )
)

;; Valeur du code dxf d'une entit (ename)

(defun val_dxf (code ent)
  (cdr (assoc code (entget ent)))
)

;;; LONGOBJT Retourne la longueur ou le primtre d'un objet

(defun LONGOBJT	(objt)
  (cond
    ((= (val_dxf 0 objt) "LINE")
     (distance (val_dxf 10 objt) (val_dxf 11 objt))
    )
    ((= (val_dxf 0 objt) "ARC")
     (* (ANGARC objt) (val_dxf 40 objt))
    )
    ((member (val_dxf 0 objt)
	     '("CIRCLE" "ELLIPSE" "LWPOLYLINE" "SPLINE")
     )
     (command "_area" "_object" objt)
     (getvar "perimeter")
    )
    ((= (type (car objt)) 'ENAME)
     (princ "\nLa longueur de cet objet n'est pas dfinie.")
    )
  )
)

;;; ANGARC Retourne l'angle dcrit par un arc de cercle (en radians).

(defun ANGARC (arc / ang)
  (setq	ang (- (val_dxf 51 arc)
	       (val_dxf 50 arc)
	    )
  )
  (if (minusp ang)
    (setq ang (+ (* 2 pi) ang))
  )
  ang
)

;;; Fonction principale

(defun C:LONG_OBJT (/ SavedSysVarLst dist)
  (setq	m:err	*error*
	*error*	LO_ERR
  )
  (command "_undo" "_begin")
  (SAVE&SET_VAR "cmdecho" 0)
  (sssetfirst nil)
  (while (not (numberp dist))
    (setq dist
	   (LONGOBJT
	     (car (entsel
		    "\nChoix de l'objet dterminant la longueur totale: "
		  )
	     )
	   )
    )
  )
  (princ dist)
  (command "_lengthen" "_total" dist)
  (command "_undo" "_end")
  (mapcar 'eval SavedSysVarLst)
  (setq	*error*	m:err
	m:err nil
  )
  (princ)
)