;;;=================================================================
;;;
;;; OOG.LSP V1.00
;;;
;;; Rorganiser les onglets
;;;
;;; Copyright (C) Patrick_35
;;;
;;;=================================================================

(defun c:oog(/ s)

  ;;;---------------------------------------------------------------
  ;;;
  ;;; Gestion des erreurs
  ;;;
  ;;;---------------------------------------------------------------

  (defun *erroog* (msg)
    (if (/= msg "Function cancelled")
      (if (= msg "quit / exit abort")
        (princ)
        (princ (strcat "\nErreur : " msg))
      )
      (princ)
    )
    (setq *error* s)
    (vla-endundomark (vla-get-activedocument (vlax-get-acad-object)))
    (princ)
  )

  ;;;---------------------------------------------------------------
  ;;;
  ;;; Routine principale
  ;;;
  ;;;---------------------------------------------------------------

  (defun organiser_onglets(/ init_oog lay lays lst n old ordre position)

    (defun affiche_liste()
      (start_list "oog")
      (mapcar 'add_list (mapcar 'vla-get-name ordre))
      (end_list)
    )

    (defun affiche_position()
      (set_tile "pos" (itoa (vla-get-taborder (nth (atoi position) ordre))))
      (set_tile "txt" "")
    )

    (defun changer_position(ou / lay loc lst)
      (if (not (zerop (setq loc (abs (atoi ou)))))
        (progn
          (set_tile "txt" "")
          (if (> loc (length ordre))
            (setq loc (length ordre))
          )
          (setq lay (nth (atoi position) ordre) ordre (vl-remove lay ordre) n 0)
          (while (< n (1- loc))
            (setq lst (append lst (list (nth n ordre))))
            (setq n (1+ n))
          )
          (setq lst (append lst (list lay)))
          (while (nth n ordre)
            (setq lst (append lst (list (nth n ordre))))
            (setq n (1+ n))
          )
          (vla-put-taborder lay loc)
          (setq position (itoa (1- loc)) ordre lst)
          (affiche_liste)
          (set_tile "oog" position)
          (affiche_position)
        )
        (progn
          (if (eq (type (read ou)) 'INT)
            (set_tile "txt" "Veuillez saisir une valeur autre que zro")
            (set_tile "txt" "Veuillez saisir une valeur numrique")
          )
          (mode_tile "pos" 2)
        )
      )
    )

    (if (findfile "oog.dcl")
      (progn
        (setq init_oog (load_dialog (findfile "oog.dcl"))
              lays (vla-get-layouts (vla-get-activedocument (vlax-get-acad-object)))
              position "0" n 1 resultat 2)
        (vlax-for lay lays
          (setq lst (append lst (list (cons (vla-get-taborder lay) lay))))
        )
        (while (assoc n lst)
          (setq ordre (append ordre (list (cdr (assoc n lst)))))
          (setq n (1+ n))
        )
        (setq old ordre)
        (new_dialog "oog" init_oog "")
        (set_tile "titre" "Rorganisation des Onglets V1.00")
        (affiche_liste)
        (set_tile "oog" "0")
        (affiche_position)
        (mode_tile "cancel" 2)
        (while (and (/= resultat 1)(/= resultat 0))
          (action_tile "oog"    "(setq position $value)(affiche_position)")
          (action_tile "pos"    "(changer_position $value)")
          (action_tile "accept" "(done_dialog 1)")
          (action_tile "cancel" "(done_dialog 0)")
          (setq resultat (start_dialog))
        )
        (unload_dialog init_oog)
        (if (and (= resultat 0) (/= old ordre))
          (progn
            (setq n 0)
            (while (nth n old)
              (vla-put-taborder (nth n old) (1+ n))
              (setq n (1+ n))
            )
          )
        )
      )
      (alert "Fichier OOG.DCL introuvable")
    )
  )

  ;;;---------------------------------------------------------------
  ;;;
  ;;; Routine de lancement
  ;;;
  ;;;---------------------------------------------------------------

  (vl-load-com)
  (setq s *error*)
  (setq *error* *erroog*)
  (vla-startundomark (vla-get-activedocument (vlax-get-acad-object)))
  (organiser_onglets)
  (vla-endundomark (vla-get-activedocument (vlax-get-acad-object)))
  (setq *error* s)
  (princ)
)

(princ "\nOOG.LSP charg. Tapez OOG pour l'excuter")
(princ)