;; TAP (gile)
;; Cre une ligne sur la tangente  la courbe au point spcifi.

(defun c:tap (/ obj)
  (vl-load-com)
  (or *acdoc*
      (setq *acdoc* (vla-get-activeDocument (vlax-get-acad-object)))
      )
  (vla-StartUndoMark *acdoc*)
  (if (and
	(setq obj (car (entsel "\nSlectionnez une courbe: ")))
	(vlax-curve-getEndParam obj)
      )
    (vl-catch-all-apply
      '(lambda (/ start par vec pt ratio line elst start loop)
	 (while	(setq start (getpoint "\nSpcifiez le point de tangence: "))
	   (if (setq par (vlax-curve-getParamAtPoint
			   obj
			   (setq start (trans start 1 0))
			 )
	       )
	     (progn
	       (setq
		 ;; vecteur directeur de la tangente
		 vec   (vunit (vlax-curve-getFirstDeriv obj par))
		 
		 ;; projection du point sur le plan du SCU
		 pt    (trans (UCSProjectAboutView start) 0 1)
		 
		 ;; rapport entre la longueur du vecteur et celle de sa projection sur le SCU
		 ratio (/ 1
			  (distance (UCSProjectAboutView '(0 0 0))
				    (UCSProjectAboutView vec)
			  )
		       )
		 ;; ligne de longueur 0
		 line  (entmakex
			 (list
			   '(0 . "LINE")
			   (cons 10 start)
			   (cons 11 start)
			 )
		       )
		 elst  (entget line)
		 loop  T
	       )
	       (princ "\nSpcifiez la longueur ou [annUler/Inverser]: ")
	       (if
		 (vl-catch-all-error-p
		   (vl-catch-all-apply
		     '(lambda (/ gr len end str)
			(while
			  (and (setq gr (grread T 12 0)) (/= (car gr) 3) loop)
			   (cond
			     ;; modification de la ligne en fonction de la position du pointeur
			     ((= 5 (car gr))
			      (setq len	(* ratio (distance pt (cadr gr)))
				    end	(mapcar
					  (function
					    (lambda (x1 x2)
					      (+ x1 (* len x2))
					    )
					  )
					  start
					  vec
					)
			      )
			      (entmod (subst (cons 11 end)
					     (assoc 11 elst)
					     elst
				      )
			      )

			      ;; affichage dynamique de la longueur dans la barre d'tat
			      (grtext -1 (rtos len))
			     )
			     
			     ;; clic droit
			     ((member (car gr) '(11 25))
			      (entdel line)
			      (setq loop nil
				    line nil
			      )
			     )
			     
			     ;; Entre ou Espace
			     ((member (cadr gr) '(13 32))
			       (cond
				 ;; longueur valide
				 ((and str (numberp (distof str)))
				  (setq	end  (mapcar
					       (function
						 (lambda (x1 x2)
						   (+ x1 (* (distof str) x2))
						 )
					       )
					       start
					       vec
					     )
					loop nil
				  )
				  (entmod (subst (cons 11 end)
						 (assoc 11 elst)
						 elst
					  )
				  )
				 )
				 
				 ;; Inverser
				 ((= (strcase str) "I")
				  (setq	vec (mapcar '- vec)
					str nil
				  )
				  (princ (chr 8))
				  (princ (chr 32))
				 )
				 
				 ;; annUler
				 ((= (strcase str) "U")
				  (entdel line)
				  (setq	loop nil
					line nil
				  )
				 )
				 
				 ;; entre non valide
				 (T
				  (princ
				    "\nNcessite un nombre valide ou une saisie au pointeur.
				     \nSpcifiez la longueur ou [annUler/Inverser]: "
				  )
				  (setq str "")
				 )
			       )
			     )
			     
			     ;; Rcupration des entre au clavier
			     (T
			      ;; retour/effacer
			      (if (= (cadr gr) 8)
				(or
				  (and
				    str
				    (/= str "")
				    (setq str (substr str 1 (1- (strlen str))))
				    (princ (chr 8))
				    (princ (chr 32))
				  )
				  (setq str nil)
				)
				(or
				  (and str
				       (setq str (strcat str (chr (cadr gr))))
				  )
				  (setq str (chr (cadr gr)))
				)
			      )

			      ;; affichage sur la ligne commande
			      (and str (princ (chr (cadr gr))))
			     )
			   )
			)
		      )
		   )
		 )
		  (and (entdel line) (setq line nil))
	       )
	     )
	   )
	 )
       )
    )
  )
  (grtext)
  (vla-EndUndoMark *acdoc*)
  (princ)
)

;;======================= SOUS ROUTINES =======================;;

;;VXV (gile)
;; Retourne le produit scalaire (rel) de deux vecteurs

(defun vxv (v1 v2)
  (apply '+ (mapcar '* v1 v2))
)


;;; VUNIT (gile)
;;; Retourne le vecteur unitaire d'un vecteur

(defun vunit (v)
  ((lambda (l)
     (if (/= 0 l)
       (mapcar (function (lambda (x) (/ x l))) v)
     )
   )
    (distance '(0 0 0) v)
  )
)

;;; ILP (gile)
;;; Retourne le point d'intersection de la droite dfinie par p1 p2
;;; et du plan dfini par un point et sa normale.

(defun ilp (p1 p2 org nor / scl)
  (if (and
	(/= 0 (setq scl (vxv nor (mapcar '- p2 p1))))
	(setq scl (/ (vxv nor (mapcar '- p1 org)) scl))
      )
    (mapcar (function (lambda (x1 x2) (+ (* scl (- x1 x2)) x1)))
	    p1
	    p2
    )
  )
)

;; UCSProjectAboutView (gile)
;; Projette un point sur le plan du SCU courant suivant la vue courante
;;
;; Argument
;; pt : le point  projeter (coordonnes SCG)
;;
;; Retour : le point sur le plan du SCU courant (coordonnes SCG)

(defun UCSProjectAboutView (pt)
  (ilp
    pt
    ((lambda (p)
       (trans
	 (list (car p) (cadr p) (1+ (caddr p)))
	 2
	 0
       )
     )
      (trans pt 0 2)
    )
    (trans '(0 0 0) 1 0)
    (trans '(0 0 1) 1 0 T)
  )
)